@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
-import sys, os
+import sys, os, time
+from columns import name as col_name, index as col_index
 
 if sys.platform == 'win32':
     #This is required in order to make pyuno usable with the default python interpreter under windows
@@ -36,17 +37,47 @@ class OODoc(object):
     This is the actual wrapper around python-uno.
     """
 
+    _macro_environment = None
+    _context = None
+    _model = None
+    _dispatcher = None
+    _cached = False
+
     def __init__(self):
-        self.macro_environment = self._detect_macro_environment()
-        self.model = self.get_model()
-        self.dispatcher = self.get_dispatcher()
-        
+        if not OODoc._dispatcher:
+            self.connect()
+        else:
+            self.load_cache()
+                
+    def connect(self):
+        start = time.time()
+        OODoc._macro_environment = self.macro_environment = self._detect_macro_environment()
+        OODoc._context = self.context = self.get_context()
+        OODoc._model = self.model = self.get_model()
+        OODoc._dispatcher = self.dispatcher = self.get_dispatcher()
+
+    def load_cache(self):
+        self.macro_environment = OODoc._macro_environment
+        self.context = OODoc._context
+        self.model = OODoc._model
+        self.dispatcher = OODoc._dispatcher
+
     def _detect_macro_environment(self):
         for layer in inspect.stack():
             if layer[1].startswith('vnd.sun.star.tdoc:'):
                 return True
         return False
     
+    def get_context(self):
+        localContext = uno.getComponentContext()
+        if self.macro_environment:
+            # We're inside openoffice macro
+            return localContext
+        else:
+            # We have to connect by socket
+            resolver = localContext.ServiceManager.createInstanceWithContext("com.sun.star.bridge.UnoUrlResolver", localContext)
+            return resolver.resolve( "uno:socket,host=localhost,port=2002;urp;StarOffice.ComponentContext" )
+        
     def get_model(self):
         """
         Desktop's current component, a pyuno object of type com.sun.star.lang.XComponent.
@@ -59,18 +90,8 @@ class OODoc(object):
         The current environment is detected to decide to connect either via socket or directly.
         
         """
-        localContext = uno.getComponentContext()
-        if self.macro_environment:
-            # We're inside openoffice macro
-            ctx = localContext
-        else:
-            # We have to connect by socket
-            resolver = localContext.ServiceManager.createInstanceWithContext("com.sun.star.bridge.UnoUrlResolver", localContext)
-            ctx = resolver.resolve( "uno:socket,host=localhost,port=2002;urp;StarOffice.ComponentContext" )
-            
-        smgr = ctx.ServiceManager
-        desktop = smgr.createInstanceWithContext( "com.sun.star.frame.Desktop",ctx)
-            
+        smgr = self.context.ServiceManager
+        desktop = smgr.createInstanceWithContext( "com.sun.star.frame.Desktop", self.context)
         return desktop.getCurrentComponent()
 
     def get_dispatcher(self):
@@ -85,15 +106,8 @@ class OODoc(object):
         
         The current environment is detected to decide to connect either via socket or directly.
         """
-        localContext = uno.getComponentContext()
-        if self.macro_environment:
-            ctx = localContext
-        else:
-            resolver = localContext.ServiceManager.createInstanceWithContext("com.sun.star.bridge.UnoUrlResolver", localContext)
-            ctx = resolver.resolve( "uno:socket,host=localhost,port=2002;urp;StarOffice.ComponentContext" )
-
-        smgr = ctx.ServiceManager
-        return smgr.createInstanceWithContext( "com.sun.star.frame.DispatchHelper", ctx)
+        smgr = self.context.ServiceManager
+        return smgr.createInstanceWithContext( "com.sun.star.frame.DispatchHelper", self.context)
 
     def args(self, name, *args):
         """
@@ -200,7 +214,7 @@ class OOSheet(OODoc):
     This high-level library works with a group of cells defined by a selector.
     """
 
-    def __init__(self, selector = None):
+    def __init__(self, selector = None, _row_sliced = False):
         """
         Constructor gets a selector as parameter. Selector can be one of the following forms:
         a10
@@ -213,7 +227,7 @@ class OOSheet(OODoc):
         Selector is case-insensitive
         """
         super(OOSheet, self).__init__()
-
+        
         if not selector:
             address = self.model.CurrentSelection.RangeAddress
             self.sheet = self.model.Sheets.getByIndex(address.Sheet)
@@ -237,7 +251,7 @@ class OOSheet(OODoc):
             (start, end) = cells.split(':')
             if not re.match('^[A-Z]', end):
                 col, row = self._position(start)
-                end = ''.join([self._col_name(col), end])
+                end = ''.join([col_name(col), end])
             self.start_col, self.start_row = self._position(start)
             self.end_col, self.end_row = self._position(end)
         else:
@@ -245,6 +259,8 @@ class OOSheet(OODoc):
             self.start_col, self.end_col = col, col
             self.start_row, self.end_row = row, row
 
+        self._row_sliced = _row_sliced
+
     @property
     def selector(self):
         """
@@ -258,8 +274,8 @@ class OOSheet(OODoc):
                                        self.start_row, self.end_row)
     
     def _generate_selector(self, start_col, end_col, start_row, end_row):
-        start = '%s%d' % (self._col_name(start_col), start_row + 1)
-        end = '%s%d' % (self._col_name(end_col), end_row + 1)
+        start = '%s%d' % (col_name(start_col), start_row + 1)
+        end = '%s%d' % (col_name(end_col), end_row + 1)
         if start != end:
             return '%s.%s:%s' % (self.sheet.Name, start, end)
         else:
@@ -288,6 +304,45 @@ class OOSheet(OODoc):
         # alias for cells property
         return self.cells
     
+    def __getitem__(self, key):
+        if isinstance(key, slice):
+            start = key.start
+            stop = key.stop
+        else:
+            start = key
+            stop = key
+
+        row_sliced = self._row_sliced
+
+        if isinstance(start, str):
+            start = col_index(start) - self.start_col
+            row_sliced = True
+        if isinstance(stop, str):
+            stop = col_index(stop) - self.start_col
+            row_sliced = True
+
+        if not row_sliced:
+            return OOSheet(self._generate_selector(self.start_col,
+                                                   self.end_col,
+                                                   self.start_row + start,
+                                                   self.start_row + stop),
+                           _row_sliced = True)
+        else:
+            return OOSheet(self._generate_selector(self.start_col + start,
+                                                   self.start_col + stop,
+                                                   self.start_row,
+                                                   self.end_row))
+
+
+    def __cmp__(self, peer):
+        return (cmp(self.sheet.Name, peer.sheet.Name) or
+                cmp(self.start_row, peer.start_row) or
+                cmp(self.start_col, peer.start_col) or
+                cmp(self.end_row, peer.end_row) or
+                cmp(self.end_col, peer.end_col))
+
+        
+
     @property
     def cells(self):
         """
@@ -305,7 +360,8 @@ class OOSheet(OODoc):
         single-cell OOSheet object        
         """
         for row in range(self.start_row, self.end_row+1):
-            yield OOSheet(self._generate_selector(self.start_col, self.end_col, row, row))
+            yield OOSheet(self._generate_selector(self.start_col, self.end_col, row, row),
+                          _row_sliced = True)
 
     @property
     def columns(self):
@@ -345,36 +401,11 @@ class OOSheet(OODoc):
         col = re.findall('^([A-Z]+)', descriptor)[0]
         row = descriptor[len(col):]
             
-        col = self._col_index(col)
+        col = col_index(col)
         row = int(row) - 1
 
         return col, row
 
-    def _col_index(self, name):
-        letters = [ l for l in name.upper() ]
-        letters.reverse()
-        index = 0
-        power = 0
-        for letter in letters:
-            index += (1 + ord(letter) - ord('A')) * pow(ord('Z') - ord('A') + 1, power)
-            power += 1
-        return index - 1
-
-    def _col_name(self, index):
-        name = []
-        letters = [ chr(ord('A')+i) for i in range(26) ]
-        
-        while index > 0:
-            i = index % 26
-            index = int(index/26) - 1
-            name.append(letters[i])
-
-        if index == 0:
-            name.append('A')
-
-        name.reverse()
-        return ''.join(name)            
-
     @property
     def basedate(self):
         """Hard-coded datetime.datetime object representing the date that corresponds to value 0"""
@@ -648,7 +679,12 @@ class OOSheet(OODoc):
         for cell in self.cells:
             if test(cell):
                 yield cell
-            
+
+    def each(self, function):
+        if type(function) is not types.FunctionType:
+            raise TypeError
+        for cell in self.cells:
+            function(cell)            
 
     def shift_right(self, num = 1):
         """Moves the selector to right, but number of columns given by "num" parameter."""
@@ -732,7 +768,7 @@ class OOSheet(OODoc):
                 ref_col = self.start_col
         else:
             assert col == 0
-            ref_col = self._col_index(position)
+            ref_col = col_index(position)
             if row > 0:
                 ref_row = self.end_row
             else:
@@ -972,3 +1008,11 @@ def print_help():
     print "Usage: %s document script.py" % script_name
     sys.exit(1)
 
+def launch():
+    print """
+# This is just a reminder of the complicated command needed to launch 
+# LibreOffice with proper parameters to be controlled by sockets
+
+  libreoffice -calc -accept="socket,host=localhost,port=2002;urp;StarOffice.ServiceManager"
+"""
+
